SEXP_CONV_PATH "Goba"

open VecOps
open GobaUtils

let startup_lasers = 5

let base_attack_sound = Sfx.Sample "base-attack"

let recharge_distance = 50.0

let recharge_sound = Sfx.Sample "recharge"

let channel = Sfx.alloc_channel ()

let max_laser_load = 10.0
let laser_accu_speed = 0.1

type init =
    { init_color : (int * int * int);
      init_at	 : float * float;
      init_hp	 : int;
      init_laser_load : float;
      init_bombs : int;
    }
with sexp

let base_bitmap_offset = (-58, -82)

class t id init0 =
  let init = Object.default_with init0 init_of_sexp in
  let color = init.init_color in
  let bitmap = lazy (Bitmaps.bitmap (Bitmaps.move (base_bitmap_offset, Bitmaps.colored (Bitmaps.file "base", color)))) in
  let cmask = 
    lazy 
      (Bitmaps.mask_with_offset Bitmaps.collision_mask_of_surface2 
	 (Bitmaps.bitmap (Bitmaps.move (base_bitmap_offset, Bitmaps.file "base-mask")))) 
  in

object (self : 'a)
  inherit Object.t id as super
  inherit Collision.t

  val mutable at = init.init_at
  val mutable hp = init.init_hp
  val mutable next_recharge = 0.0
  val mutable under_attack_sound = false
  val mutable laser_load = init.init_laser_load
  val mutable laser_enabled = false
  val mutable bombs = init.init_bombs

  method get_type = ObjectType.Base

  method get_laser_load =
    laser_load

  method get_bombs =
    bombs

  method get_state delay =
    Object.sexp_of_state sexp_of_init
      { Object.state_super = super#get_state delay;
	state_init = { init_color = color;
		       init_at = at;
		       init_hp = hp;
		       init_laser_load = laser_load;
		       init_bombs = bombs } }

  method set_state state delay =
    let state = Object.state_of_sexp init_of_sexp state in
      super#set_state state.Object.state_super delay;
      let init = state.Object.state_init in
	hp <- init.init_hp;
	laser_load <- init.init_laser_load

  method set_location at' = at <- at'

  method location = at
  method set_speed speed' = ()
  method speed = 0.0, 0.0

  (* FIXME: what really needs to be fixed is the tank bitmap
     location.. (but it would only partially compensate *)
  method z_location = 25.0

  method render (gfx:#Gfx.t) (_:Object.render_flags) at = 
    let ofs, surface = Lazy.force bitmap in
      (*    let ofs = ofs +| bitmap_offset in*)
      (*( for xc = 0 to fst layout_size - 1 do
	    ( for yc = 0 to snd layout_size - 1 do
		  if layout.(yc).(xc) = 1 then
		    gfx#blit (at +| ofs +| (xc, yc) *| tile) surface
	      done )
	done )*)
      gfx#blit (at +| ofs) surface

  method do_recharge (world:Object.world) =
    if self#self_owned then
      (*let at0 = at +|. Vector.float2 bitmap_offset in*)
      let at0 = at in
      let objs = 
	world#objs 
	  (at0 -|. Vector.v2 recharge_distance, at0 +|. Vector.v2 recharge_distance)
	  (fun o -> o#self_owned && Vector.abs2 (o#location -|. at0) < recharge_distance)
      in
	if List.fold_left (fun recharged o -> o#recharge || recharged) false objs then
	  if not (world#channel channel)#playing then
	    (world#channel channel)#play recharge_sound

  method tick delta (world:Object.world) = 
    if under_attack_sound then
      begin
	under_attack_sound <- false;
	if not (world#channel channel)#playing then
	  (world#channel channel)#play ~loops:(-1) base_attack_sound;
	(world#channel channel)#expire (Some 2.0);
      end;
    if laser_enabled then
      laser_load <- max 0.0 (laser_load -. delta)
    else
      laser_load <- min max_laser_load (laser_load +. laser_accu_speed *. delta);
    next_recharge <- next_recharge -. delta;
    if next_recharge <= 0.0 then
      begin
	next_recharge <- 0.2;
	self#do_recharge world;
      end

  method move f = 
    at <- f at

  method is_alive = hp > 0

  method collision_area = 
    let ofs, mask = Lazy.force cmask in
    let at0 = at +|. Vector.float2 ofs in
    let size = Vector.float2 (CollisionMask.size mask) in
      (*Printf.printf "Collision area: %f,%f + %f,%f\n%!" (fst at0) (snd at0) (fst size) (snd size);*)
      (at0, at0 +|. size), mask

  method get_hp = hp

  method signal_bomb =
    if bombs > 0 then
      begin
	bombs <- bombs - 1;
	self#change;
	true;
      end
    else
      false

  method signal_laser state =
    if laser_load > 0.1 && state then
      begin
	if not laser_enabled then
	  begin
	    laser_enabled <- state;
	    self#change;
	  end;
	true
      end
    else
      begin
	if laser_enabled then
	  begin
	    laser_enabled <- false;
	    self#change;
	  end;
	false
      end

  method cause_damage damage =
    under_attack_sound <- true;
(*    if not (world.Object.world.world_channel channel)#playing then
    (*	    Sdlmixer.play_channel ~channel:7 (Lazy.force recharge_sound)

      Sdlmixer.play_channel ~loops:~-1 ~channel:7 (Lazy.force base_attack_sound);*)
(*    Sdlmixer.expire_channel 7 (Some 2.0);*)
    (world.Object.world.world_channel channel)#expire (Some 2.0);*)
    hp <- max 0 (hp - damage);
    self#change

  initializer
    (match init0 with 
       | Object.Init _ -> ()
       | Object.State (state0, age) -> self#set_state state0 age);
    self#change;
end
