#!/usr/bin/perl -w

use strict;

my $dbdir = "/home/flux/goba-meta";
my $update_interval = 60;

my $remote = $ENV{REMOTE_ADDR};

sub decode {
  my ($msg) = @_;
  $msg =~ tr/+/ /;
  $msg =~ s/%([0-9a-f]{2})/chr(hex($1))/ge;
  return $msg;
}

# sexp parsing functions.
sub scan_atom;
sub string_of_sexp;

sub skip_ws {
  #print "ws\n";
  my ($str, $at) = @_;
  while ($at < length($str) && substr($str, $at, 1) =~ /[\n \t]/) {
    ++$at;
  }
  return $at;
}

sub scan_string {
  my ($str, $at0) = @_;
  my $at = $at0 + 1;
  #print "string begin at $at0 with ", substr($str, $at0, 1), "\n";
  my $in_string = 1;
  my $escape = 0;
  #print "begun at $at0 with ", substr($str, $at0, 1), "\n";
  while ($in_string) {
    #print "$at..\n";
    if ($at >= length($str)) {
      die "invalid string";
    }
    my $ch = substr($str, $at, 1);
    if ($escape) {
      $escape = 0;
    } else {
      if ($ch eq "\\") {
        $escape = 1;
      } elsif ($ch eq "\"") {
        $in_string = 0;
      }
    }
    ++$at;
  }
  my $at1 = $at;
  $at = skip_ws $str, $at;
  my $str2 = substr($str, $at0, $at1 - $at0);
  #print "begun at $at0 with ", substr($str, $at0, 1), "\n";
  #print "string end at $at: $str", substr($str, $at0, 1), "\n";
  #print "string=$str2\n";
  return ($str2, $at);
}

sub scan_list {
  my ($str, $at0) = @_;
  if (substr($str, $at0, 1) eq ")") {
    return ([], $at0 + 1);
  } else {
    my ($elem, $at1) = scan_atom $str, $at0;
    my $at = skip_ws $str, $at1;
    #print "list: $elem\n";
    my $list;
    ($list, $at) = scan_list($str, $at);
    #print "whole list: ", string_of_sexp([$elem, @$list], 0, 0), "\n";
    return ([$elem, @$list], $at);
  }
}

sub scan_ident {
  #print "ident\n";
  my ($str, $at0) = @_;
  my $at = $at0;
  while ($at < length($str) && substr($str, $at, 1) =~ /[^ ()\"]/) {
    ++$at;
  }
  my $at1 = $at;
  my $str2 = substr($str, $at0, $at1 - $at0);
  #print "ident $str\n";
  $at = skip_ws $str, $at;
  return ($str2, $at);
}

sub scan_atom {
  #print "atom\n";
  my ($str, $at) = @_;
  if ($at >= length($str)) {
    die "invalid sexp";
  }
  my $ch = substr($str, $at, 1);
  if ($ch eq "(") {
    return scan_list($str, $at + 1);
  } elsif ($ch eq "\"") {
    return scan_string($str, $at);
  } elsif ($ch eq ")") {
    return $at;
  } else {
    return scan_ident($str, $at);
  }
}

sub sexp_of_string {
  my ($str, $at) = @_;
  my $root;
  ($root, $at) = scan_atom $str, $at;
  if ($at != length($str)) {
    die "sexp not finished";
  }
  return ($root);
}

sub string_of_sexp {
  my ($root, $indent, $newlines, $want_separator) = @_;
  my $str = "";
  if (!defined $want_separator) {
    $want_separator = 0;
  }
  if (!defined $newlines) {
    $newlines = 1;
  }
  if (!defined $indent) {
    $indent = 0;
  }
  if (ref $root eq "ARRAY") {
    if ($newlines) {
      $str .= ((" " x $indent) . "(\n");
    } else {
      $str .= "(";
    }
    my $separator = 0;
    map { $str .= string_of_sexp ($_, $indent + 2, $newlines, $separator); $separator = 1; } @{$root};
    if ($newlines) {
      $str .= ((" " x $indent) . ")\n");
    } else {
      $str .= ")";
    }
  } else {
    if ($newlines) {
      $str .= ((" " x $indent) . $root . "\n");
    } else {
      if ($want_separator) {
        $str .= " ";
      }
      $str .= "$root";
    }
  }
  return $str;
}

sub header {
  my ($type, $length) = @_;
  print "Content-Type: text/plain\r\n";
  if (defined ($length)) {
    print "Content-Length: $length\r\n";
  }
  print "\r\n";
}

sub assoc_of_sexp {
  my ($root) = @_;
  die "not a list" if ref $root ne "ARRAY";
  my %assoc;
  foreach my $el (@{$root}) {
    $assoc{$el->[0]} = $el->[1];
  }
  return %assoc;
}

sub sexp_of_assoc {
  my (%assoc) = @_;
  my @root;
  foreach my $k (keys %assoc) {
    push @root, [$k, $assoc{$k}];
  }
  return [@root];
}

sub is_number {
  my ($n) = @_;
  return $n =~ /^[0-9]+$/;
}

sub register {
  my ($data) = @_;
  my $sexp;
  eval {
    $sexp = sexp_of_string($data);
  };
  if ($@) {
    print STDERR "Error parsing data: $@\n"
  } else {
    my %info = assoc_of_sexp $sexp;
    my $port = $info{server_addr}->[1];
    if (!is_number $port) {
      print STDERR "$port is not a port number\n";
    } else {
      $info{server_addr} = [$remote, $port];
      my $fname = "$dbdir/$remote:$port";
      open OUT, ">", $fname or die "Failed to open $fname";
      print OUT string_of_sexp(sexp_of_assoc(%info), 0, 0);
      close OUT;
    }
  }
  my %update = (update_interval => $update_interval);
  my $buffer = string_of_sexp(sexp_of_assoc(%update), 0, 0);
  header("text/plain", length($buffer));
  print $buffer;
}

sub unregister {
  my ($data) = @_;
  my $sexp;
  eval {
    $sexp = sexp_of_string($data);
  };
  if ($@) {
    print STDERR "Error parsing data: $@\n";
  } else {
    my %info = assoc_of_sexp $sexp;
    my $host = $remote;
    my $port = $info{server_addr}->[1];
    if (!is_number $port) {
      print STDERR "$port is not a port number\n";
    } else {
      my $fname = "$dbdir/$remote:$port";
      unlink $fname;
    }
  }
  header("text/plain", 0);
}

sub post {
  my $input = <>;
  chomp $input;
  my %fields = map { if (/^([^=]*)=(.*)/) { ($1 => decode($2)) } else { "" => "" } } split(/\&/, $input);
  if (exists $fields{data}) {
    register $fields{data};
  } elsif (exists $fields{unregister}) {
    unregister $fields{unregister};
  }
}

sub get {
  my @servers = <$dbdir/*>;
  my $buffer = "";
  my $now = time();
  $buffer .= "(";
  foreach my $server (@servers) {
    if ($now - (stat($server))[9] < 2 * $update_interval) {
      open(F, "<", $server);
      my $l = <F>;
      $buffer .= $l;
      close(F);
    }
  }
  $buffer .= ")";
  header("text/plain", length($buffer));
  print $buffer;
}

################################################################################
# Main thread of execution begins here

# doesn't support resolving or ipv6..
if ($remote =~ /^[0-9]+.[0-9]+.[0-9]+.[0-9]+$/) {
  # ok
} else {
  $remote = undef;
}


if (!defined $remote) {
  header("text/plain", undef);
  printf "Remote unknown: $ENV{REMOTE_ADDR}\n";
  exit 0;
}

my $request_method = $ENV{REQUEST_METHOD};
if (!defined $request_method) {
  $request_method = "POST";
}

if ($request_method eq "GET") {
  get();
} elsif ($request_method eq "POST") {
  post();
} else {
  my $buffer = "unsupported request method: $request_method";
  print header("text/plain", length($buffer));
  print $buffer;
}
