SEXP_CONV_PATH "Goba"

module Http = Http_client.Convenience

type t = { url : string }

type address = (Unix.inet_addr * int)

let address_of_sexp sexp = 
  let module C = Sexplib.Conv in
    match sexp with
    | Sexplib.Sexp.List [address; port] ->
	(Unix.inet_addr_of_string (C.string_of_sexp address), 
	 C.int_of_sexp port)
    | _ -> raise (C.Of_sexp_error ("address_of_sexp", sexp))

let sexp_of_address (address, port) = 
  let module C = Sexplib.Conv in
  Sexplib.Sexp.List 
    [C.sexp_of_string (Unix.string_of_inet_addr address);
     C.sexp_of_int port]
    
type server = { server_addr : address;	(* server address and port *)
		server_version : int;	(* which release is the server running *)
		server_name : string;	(* server name *)
		server_message : string; (* server message *)
		server_players : int;	(* how many players are on *)
		server_map_dims : int * int; (* map dimensions *)
		server_starttime : float;	(* how many seconds has the server been up *)
	      }
with sexp

type update = { update_interval : int;	(* you are expected to refresh the information every n seconds *)
	      }
with sexp

(* set up a metaserver client *) 
let create : string -> t = 
  fun url ->
    { url = url }

(* retrieve a list of servers available *)
let list_servers : t -> server list =
  fun { url = url } ->
    let response = Http.http_get url in
      match Sexplib.Sexp.of_string response with
	| Sexplib.Sexp.List l ->
	    List.concat
	      (List.map	
		 (fun sexp ->
		    try [server_of_sexp sexp]
		    with Sexplib.Conv.Of_sexp_error _ -> []
		 ) l)
	| Sexplib.Sexp.Atom _ ->
	    failwith "expected list; received atom"
	
(* update server info *)
let update_server : t -> server -> update =
  fun { url = url } info ->
    let request = Sexplib.Sexp.to_string (sexp_of_server info) in
    let response = Http.http_post url [("data", request)] in
      update_of_sexp (Sexplib.Sexp.of_string response)

(* remove registration info *)
let unregister_server : t -> server -> unit =
  fun { url = url } info ->
    let request = Sexplib.Sexp.to_string (sexp_of_server info) in
      ignore (Http.http_post url [("unregister", request)])
